/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvpolyline.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVPOLYLINE_H
#define QVPOLYLINE_H

#include <qvipp/qvipp.h>
#include <qvcore/qvimage.h>

namespace qvdta
{
/// @class QVPolyline qvdta/qvpolyline.h
/// @brief This class represents polylines.
///
/// A polyline is a continuous line composed of one or more straight line segments.
/// A QVPolyline is a implementation of a polyline, specified by a list of the points located at the intersection of the segments.
/// Thus, any QVPolyline object inherits from the class QList<QPoint>.
///
/// When a polyline is closed, then it is considered to be a polygon. In that case it is supossed to have an implicit final
/// straight line segment, that links the first and the last point in the polyline. The property @ref QVPolyline::closed will
/// store the value <i>TRUE</i> if the polyline is a polygon, else storing <i>FALSE</i>.
///
/// A QVPolyline has the property @ref QVPolyline::direction, that will indicate if it is a direct polyline, or a reverse
/// polyline. It has only meaning for some algorithms, like @ref getBorderContourThreshold.
/// @ingroup qvdta
class QVPolyline: public QList<QPoint>
	{
	public:
		bool closed, direction;

		QVPolyline();
		QVPolyline(const QVPolyline &polyline);

		static QVPolyline ellipse(uInt n, float x, float y, float maxRadio, float minRadio, float ang);
		static QVPolyline line(int x1, int y1, int x2, int y2);
		static QVPolyline rectangle(int x1, int y1, int x2, int y2);
	};

void draw(QVImage<uChar> &image, const QVPolyline &polyline, const uChar constant, bool linked=FALSE, bool safe=FALSE);
void draw(QVImage<uChar,3> &image, const QVPolyline &polyline, const uChar constant[3], bool linked=FALSE, bool safe=FALSE);
void draw(QVImage<uChar> &img, const QList< QVPolyline > &polylineList, const uChar constant, bool linked=FALSE, bool safe=FALSE);
void draw(QVImage<uChar,3> &img,const QList< QVPolyline > &polylineList,const uChar constant[3],bool linked=FALSE,bool safe=FALSE);

void drawPoints(const QList<QPoint> &hotPoints, QVImage<uChar> &dest);
void drawPoints(const QList<QPoint> &hotPoints, QVImage<uChar,3> &dest);
}
#endif

