/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvisioninterface.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvisioninterface.h>

QVisionInterface::QVisionInterface(QWidget *parent): QWidget(parent),qvimagecanvaslist()
	{
	qDebug() << "QVisionInterface::QVisionInterface()";

	if(qvApp == NULL)
		{
		QString str = "QVisionInterface::QVisionInterface(): "
						": the QVisionInterface cannot be created before the "
						"QVApplication instance. Aborting now.";
		std::cerr << qPrintable(str) << std::endl;
		exit(1);
		}

	connect(this,SIGNAL(closed()),qvApp,SLOT(quit()));

	form.setupUi(this);

	leftVBox = form.vboxLayout;
	setWindowTitle("QVisionInterface");

	qDebug() << "QVisionInterface::QVisionInterface(): connected slots newFrame() <--> showImage()";

	QSetIterator<QVPropertyHolder *> iw(qvApp->getQVPropertyHolders());
	while (iw.hasNext())
		{
		QVPropertyHolder* qvp = iw.next();
		QVWorker* worker;
		QVCamera* cam;
		if((worker = dynamic_cast<QVWorker*>(qvp)) != NULL)
			add(worker);
		else if((cam = dynamic_cast<QVCamera*>(qvp)) != NULL)
			add(cam);
		}
	qvApp->registerGUI(this);
	show();
	qDebug() << "QVisionInterface::QVisionInterface() <~ return";
	}

QVisionInterface::~QVisionInterface()
	{
	QListIterator<QVImageCanvas*> i(qvimagecanvaslist);
	while (i.hasNext())
	delete(i.next());
	}

void QVisionInterface::closeEvent(QCloseEvent *event)
	{
	Q_UNUSED(event);
	emit closed();
	}

void QVisionInterface::add(QVCamera *camera)
	{
	QVCameraWidgetSmall *cameraWidget = new QVCameraWidgetSmall(camera);
	leftVBox->addWidget(cameraWidget);
	}

void QVisionInterface::add(QVWorker * worker)
	{
	// Adding params
	QVParamsInspectorWidget * widget2 = new QVParamsInspectorWidget(worker);
	form.toolBox->addItem(widget2,worker->getName());

	// Adding worker interface small
	QVWorkerInterfaceSmall *workerWidget = new QVWorkerInterfaceSmall(worker);
	leftVBox->addWidget(workerWidget);
	}

bool QVisionInterface::setVisible(QVWorker *worker,QString prop_orig,float low,float high)
	{
	QVImageCanvas *img_canvas;

	if(worker->isType<QVImage<uChar,1> >(prop_orig))
		{
 			img_canvas = new QVImageCanvas(prop_orig,FALSE);
			worker->linkProperty(prop_orig,img_canvas,"imageGray",QVWorker::AsynchronousLink);
		}
	else if(worker->isType<QVImage<sFloat,1> >(prop_orig))
		{
 			img_canvas = new QVImageCanvas(prop_orig,FALSE,TRUE);
			img_canvas->setLowHigh(low,high);
			worker->linkProperty(prop_orig,img_canvas,"imageGrayFloat",QVWorker::AsynchronousLink);
		}
	else if(worker->isType<QVImage<uChar,3> >(prop_orig))
		{
 			img_canvas = new QVImageCanvas(prop_orig,TRUE);
			worker->linkProperty(prop_orig,img_canvas,"imageRGB",QVWorker::AsynchronousLink);
		}
	else
		{
		std::cerr << "Warning: something incorrect in QVisionInterface::setVisible(): type of property " << qPrintable(prop_orig) << " should be QVImage<uChar,1>, QVImage<uChar,3>, or QVImage<sFloat,1>\n";
		return FALSE;
		}
	img_canvas->show();

	QObject::connect(worker,SIGNAL(endIteration()),img_canvas,SLOT(refreshImageArea())/*,Qt::BlockingQueuedConnection*/);
	qvimagecanvaslist.push_back(img_canvas);
	return TRUE;
	}

